/*      randomdot.c is a program to convert a ppm image file that contains
|*      the simple structure of an image to be made 3d.  I generate this
|*      file by exporting IslandPaint as sun raster.  Then I run
|*      rasttopnm to get the ppm.  (rasttopnm is in the pbmplus distribution)
|*      I then run this prog with:
|*      randomdot < infile > outfile
|*
|*      So what is this for anyway?  This creates what is know as a
|*      Single Image Random Dot Sterogram.  Say What?  I you unfocus your
|*      eyes and stare at the image (right side up) long enough all will
|*      become clear.  Times for focusing on the new image vary between
|*      1 min to 15 min.  Have patience and you will be rewarded.
|*
|*      Acknowledgments:
|*      Pentica Systems INC In-circuit Emulators
|*      One Kendall Square
|*      Building 200
|*      Cambridge, MA 02139
|*      for suppling the pamphlet within which I learned the miracle of
|*      Random Dot Sterograms.
|*
|*      Stero World Magazine.
|*      For running 2 articles about random dot sterograms in their
|*      March/April 1990
|*      and some other month i can't find right no.
|*
|*      And Tom (not sure of his last name) at PENTICA for writing some
|*      vga display code.  I served well as an example of the algorithm.
|*      without which this would not have been written.
|*
|*      History:
|*      1/23/92:        started trying to read the rasterfile.
|*      1/24/92:        used ppm instead.  finished first test at 11:30 pm.
|*      1/27/92:        now supports multiple levels of depth.
|*	17/2/92:	Optimization and color support added by 
|*			Jouni Pekkanen <jone@vipunen.hut.fi> (Thank you)
|*
|*
|*      More to come i promise.  It will support multiple levels of depth.
|*      send comments, corrections, improvements, suggestions, pizza crusts,
|*      etc... to
|*      long1@eng.auburn.edu
|*
|*      Also I must thank my boss for not sacking me for wasting my time
|*      with this.
*/

#include <stdio.h>
#define PPM_MAGIC6 "P6"
#define VALID_OPS "s:d:c:S:iv:"
#define USAGE_MSG "Usage: %s [-s trip] [-d epth] [-c 0=b/w, 1=gray, 2=color]\n\
           [-S rand() seed] [-i nverse] [-v erbosity 0-2]\n",argv[0]

#ifndef lint
static char SCCS_id[] = "%Z%%M%  %I%  %G%    Auburn University Engineering";
#endif

/*	My interpretation of the PPM standard header
*/
struct ppm {
	    char magic[2];
	    int width;
	    int height;
	    int maxrgb;
	    };

struct ppm headerin;
int strip_width,depth_factor,color_mode=2,divider,offset;
char inverse=0;
char verbose=2;
unsigned char *image;

void process_args();
void read_image();
void write_image();
void build_strip_bw();
void build_strip_g();
void build_strip_c();
void make_3d();
void make_3d_i();
int foreground();

main (argc,argv)
int argc;
char *argv[];
{
	FILE *in,*out;

	in=stdin;
	out=stdout;
	strip_width=64;
	depth_factor=16;

	process_args(argc,argv);

	fscanf(in,"%s",headerin.magic);
	if( strcmp(headerin.magic,PPM_MAGIC6) ) 
		fprintf(stderr,"Fatal:  PPM magic %s is not yet supported.\n",headerin.magic),exit(-3);
	fscanf(in,"%d",&headerin.width);
	fscanf(in,"%d",&headerin.height);
	fscanf(in,"%d ",&headerin.maxrgb);
	fprintf(stderr,"Size of input image is %dx%d = %d pixels\n",headerin.width,
		headerin.height,headerin.width*headerin.height);
	if ( !(image = (unsigned char *)calloc((unsigned) headerin.width*headerin.height,3)) )
		fprintf(stderr,"Fatal: Could not calloc needed storage!!\n"),exit(-2);

	if (verbose) fprintf(stderr,"Using a Strip of %d and a Depth of %d\n",strip_width,depth_factor);
	if ((strip_width*depth_factor)<256) { fprintf(stderr,"Depth %d? Hmm...\n",depth_factor); }
	divider=3*depth_factor;
	offset=strip_width-(256/depth_factor);
	read_image(in,headerin.width,headerin.height);

	switch (color_mode)
	{
		case 0:
			build_strip_bw(strip_width,headerin.width,headerin.height);
			break;
		case 1:
			build_strip_g(strip_width,headerin.width,headerin.height);
			break;
		default:
			build_strip_c(strip_width,headerin.width,headerin.height);
			break;
	}
			
   if (inverse) make_3d_i(strip_width,depth_factor,headerin.width,headerin.height);
	else make_3d(strip_width,depth_factor,headerin.width,headerin.height);
   write_image(out,headerin.width,headerin.height);
   cfree(image);
   exit(0);
}

void process_args(argc,argv)
int argc;
char *argv[];
{
	extern char *optarg;
	extern int   optind;
	int   flag;

	while (( flag=getopt(argc,argv,VALID_OPS)) != -1)
	switch ( flag )
	{
		case 's':
			strip_width = atoi(optarg);
			break;
		case 'd':
			depth_factor = atoi(optarg);
			break;
		case 'c':
			color_mode = atoi(optarg);
			break;
		case 'S':
			srand(atoi(optarg)); srandom(atoi(optarg));
			break;
		case 'i':
			inverse=!inverse;
			break;
		case 'v':
			verbose=atoi(optarg);
			if (verbose) fprintf(stderr,"Verbosity level %d\n",verbose);
			break;
		default:
			fprintf(stderr,USAGE_MSG);
			exit(1);
			break;
   }
}

/* 	Read_Image reads from an already open data file into an
|*	array of image_pixel structures.
|*	PARAMETERS:
|*	file_ptr:		A pointer to an open file that has been read
|*				past the header information. ( IN OUT)
|*	image_ptr:		A pointer to an already calloc area of memory,
|*				this will hold the image. ( IN OUT)
|*	width:			The images width. (IN)
|*	height:			The images height. (IN)
|*	RETURNS:
|*	nothing
*/

void read_image(file_ptr,width,height)
FILE *file_ptr;
int width,height;
{
	int pixels;

	if(verbose)	fprintf(stderr,"Reading Image.\n");
	pixels=fread(&image[0],3,width*height,file_ptr);
	if(verbose>1) fprintf(stderr,"%d pixels read.\n",pixels);

}

/*	Build_Strip is used to create random column of dots down the 
|*	right side of the image.  I call this the 3d seed.
|*	PARAMETERS:
|*	image_ptr:		A pointer to the image array ( IN OUT )
|*	width:			width of 3d seed ( IN )
|*	image_width:		The width of the image to be converted ( IN )
|*	image_height:		The number of rows in the column ( IN )
*/

void build_strip_bw(width,image_width,image_height)
int width,image_width,image_height;
{
	int x,y;
	unsigned char *i;
	extern random();

	if (verbose) fprintf(stderr,"Building B/W Strip.\n");
	
	for(y=0;y<image_height;y++)
	{
		i=&image[y*image_width*3];
		if (verbose>1) { if (!(y%10)) { fprintf(stderr,"\015Line %d",y); } }
    	for(x=0;x<width;x++)
    	{
			if (random()%3)
	    {
	    *(i++)=255;
	    *(i++)=255;
	    *(i++)=255;
	    }
	 else
	    {
	    *(i++)=0;
	    *(i++)=0;
	    *(i++)=0;
	    }
      }
   }
	if (verbose>1) fprintf(stderr,"\015");
}
void build_strip_g(width,image_width,image_height)
int width,image_width,image_height;
{
	int x,y;
	unsigned char *i;
	unsigned char tmp;
	extern random();

	if (verbose) fprintf(stderr,"Building Grey-Scale Strip.\n");
	for(y=0;y<image_height;y++)
	{
		i=&image[y*image_width*3];
		if (verbose>1) { if (!(y%10)) { fprintf(stderr,"\015Line %d",y); } }
		for(x=0;x<width;x++)
		{
			tmp=random()%256;
			*(i++)=tmp;
			*(i++)=tmp;
	 		*(i++)=tmp;
		}
	}
	if (verbose>1) fprintf(stderr,"\015");
}
void build_strip_c(width,image_width,image_height)
int width,image_width,image_height;
{
	int x,y;
	unsigned char *i;
	extern random();

	if (verbose) fprintf(stderr,"Building Color Strip.\n");
	for(y=0;y<image_height;y++)
	{
		i=&image[y*image_width*3];
		if (verbose>1) { if (!(y%10)) { fprintf(stderr,"\015Line %d",y); } }
		for(x=0;x<width;x++)
		{
			*i++=random()%256;
			*i++=random()%256;
			*i++=random()%256;
		}
	}
	fprintf(stderr,"\015");
}

/*	Make_3d is the meat of the program.  It decides wether the current
|*	pixel is in the forground.  If so look in the previous column + shift
|*	for the color value.
|*	PARAMETERS:
|*	image_ptr:		The pointer to the image to be processed(IN OUT)
|*	width:			The column width of the strip (IN)
|*	shift:			The value used to compute the pixel placment(IN)
|*	image_width:		The width of the image. (IN)
|*	image_height:		The height of the image. (IN)
|*	RETURNS:
|*	nothing
*/

void make_3d_i(width,shift,image_width,image_height)
int width,shift,image_width,image_height;
{
register int x,y,line;
register unsigned char tmp;
register unsigned char *i,*j;

	if (verbose) fprintf(stderr,"Processing Image.\n");
	for(y=0;y<image_height;y++)
	{
		if (verbose>1) { if(!(y%10)) { fprintf(stderr,"\015Line %d",y); } }
		line=y*image_width*3;
		for(x=width;x<image_width;x++)
		{
			i=&image[x*3+line];
			tmp=(i[0]+i[1]+i[2])/divider;
			j=&image[line+(x-width+tmp)*3];
			i[0]=j[0];
			i[1]=j[1];
			i[2]=j[2];
		}
	}
	if (verbose>1) fprintf(stderr,"\015");
}

void make_3d(width,shift,image_width,image_height)
int width,shift,image_width,image_height;
{
register int x,y,line;
register unsigned char tmp;
register unsigned char *i,*j;

	if (verbose) fprintf(stderr,"Processing Image.\n");
	for(y=0;y<image_height;y++)
	{
		if (verbose>1) { if(!(y%10)) { fprintf(stderr,"\015Line %d",y); } }
		line=y*image_width*3;
		for(x=width;x<image_width;x++)
		{
			i=&image[x*3+line];
			tmp=(i[0]+i[1]+i[2])/divider;
			j=&image[line+(x-tmp-offset)*3];
			*i++=*j++;
			*i++=*j++;
			*i++=*j++;
		}
	}
	if (verbose>1) fprintf(stderr,"\015");
}

/*	Foregournd determines the depth of the current pixel.  presently this
|*	this is olny 0 or 1.  In the future this will return a range of values
|*	which will be used to compute the shift offset in make_3d.
|*	PARAMETERS:
|*	image_ptr:		Pointer to the image array (IN)
|*	x:			x coord of point of intreset (IN)
|*	y:			y coord of point of intreset (IN)
|*	image_width:		the image witdh (IN)
|*	RETURNS:
|*	a value between 0 and depth_factor: this is computed by taking the
|*	average rgb value and making it a percent of the total depth. 
|*	it is ordered such that dark colors are farther out of the page 
|*	than lighter ones.
*/

/*int foreground(x,y,image_width)
register int x,y,image_width;
{
	int	depth;
	unsigned char tmp[3];

	tmp = &image[(x+y*image_width)*3];
	depth = (tmp[0]+tmp[1]+tmp[2])/divider;
	return(depth);
}*/

/*      Read_Image reads from an already open data file into an
|*      array of image_pixel structures.
|*      PARAMETERS:
|*      file_ptr:               A pointer to an open file to write the output
|*				to (ppm) (IN OUT)
|*	image_ptr:		A pointer to the image array for writing 
|*				the output from. (IN)
|*      width:                  The images width. (IN)
|*      height:                 The images height. (IN)
|*      RETURNS:
|*      nothing
*/

void write_image(file_ptr,width,height)
FILE *file_ptr;
int width,height;
{
	int pixels;

	fprintf(file_ptr,"%s\n",PPM_MAGIC6);
	fprintf(file_ptr,"%d %d\n",width,height);
	fprintf(file_ptr,"%d\n",255);
	if (verbose) fprintf(stderr,"Writing Image.\n");

   
	pixels=fwrite(&image[0],3,width*height,file_ptr);
	if (verbose>1) fprintf(stderr,"Wrote %d pixels.\n",pixels);
}

